@@ -1,12 +1,17 @@
-from .config import BaseBridgeConfig
-from .custom_puppet import (CustomPuppetMixin, CustomPuppetError, OnlyLoginSelf,
-                            InvalidAccessToken, AutologinError, OnlyLoginTrustedDomain,
-                            HomeserverURLNotFound)
-from .matrix import BaseMatrixHandler
-from .portal import BasePortal
-from .user import BaseUser
-from .puppet import BasePuppet
-from .bridge import Bridge
-from .notification_disabler import NotificationDisabler
 from .async_getter_lock import async_getter_lock
-from mautrix.util.bridge_state import BridgeState
+from .bridge import Bridge
+from .config import BaseBridgeConfig
+from .custom_puppet import (
+    AutologinError,
+    CustomPuppetError,
+    CustomPuppetMixin,
+    HomeserverURLNotFound,
+    InvalidAccessToken,
+    OnlyLoginSelf,
+    OnlyLoginTrustedDomain,
+)
+from .matrix import BaseMatrixHandler
+from .notification_disabler import NotificationDisabler
+from .portal import BasePortal
+from .puppet import BasePuppet
+from .user import BaseUser
