@@ -3,12 +3,19 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Union
+from __future__ import annotations
 
-from mautrix.errors import MatrixResponseError
 from mautrix.api import Method, Path
-from mautrix.types import (UserID, LoginType, UserIdentifier, LoginResponse, LoginFlowList,
-                           MatrixUserIdentifier, WhoamiResponse)
+from mautrix.errors import MatrixResponseError
+from mautrix.types import (
+    LoginFlowList,
+    LoginResponse,
+    LoginType,
+    MatrixUserIdentifier,
+    UserID,
+    UserIdentifier,
+    WhoamiResponse,
+)
 
 from .base import BaseClientAPI
 
@@ -39,11 +46,17 @@ class ClientAuthenticationMethods(BaseClientAPI):
         except KeyError:
             raise MatrixResponseError("`flows` not in response.")
 
-    async def login(self, identifier: Optional[Union[UserIdentifier, UserID]] = None,
-                    login_type: LoginType = LoginType.PASSWORD, device_name: Optional[str] = None,
-                    device_id: Optional[str] = None, password: Optional[str] = None,
-                    store_access_token: bool = True, update_hs_url: bool = False, **kwargs: str
-                    ) -> LoginResponse:
+    async def login(
+        self,
+        identifier: UserIdentifier | UserID | None = None,
+        login_type: LoginType = LoginType.PASSWORD,
+        device_name: str | None = None,
+        device_id: str | None = None,
+        password: str | None = None,
+        store_access_token: bool = True,
+        update_hs_url: bool = False,
+        **kwargs: str,
+    ) -> LoginResponse:
         """
         Authenticates the user, and issues an access token they can use to authorize themself in
         subsequent requests.
@@ -78,11 +91,15 @@ class ClientAuthenticationMethods(BaseClientAPI):
             kwargs["device_id"] = device_id
         elif self.device_id:
             kwargs["device_id"] = self.device_id
-        resp = await self.api.request(Method.POST, Path.login, {
-            "type": str(login_type),
-            "identifier": identifier.serialize(),
-            **kwargs,
-        })
+        resp = await self.api.request(
+            Method.POST,
+            Path.login,
+            {
+                "type": str(login_type),
+                "identifier": identifier.serialize(),
+                **kwargs,
+            },
+        )
         resp_data = LoginResponse.deserialize(resp)
         if store_access_token:
             self.mxid = resp_data.user_id
@@ -91,8 +108,10 @@ class ClientAuthenticationMethods(BaseClientAPI):
         if update_hs_url:
             base_url = resp_data.well_known.homeserver.base_url
             if base_url and base_url != self.api.base_url:
-                self.log.debug("Login response contained new base URL, switching from "
-                               f"{self.api.base_url} to {base_url}")
+                self.log.debug(
+                    "Login response contained new base URL, switching from "
+                    f"{self.api.base_url} to {base_url}"
+                )
                 self.api.base_url = base_url.rstrip("/")
         return resp_data
 
