@@ -1,22 +1,26 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Tuple
-import warnings
-import asyncio
+from __future__ import annotations
+
 import json
 
+from aiohttp import ClientError, ClientSession, ContentTypeError
 from yarl import URL
-from aiohttp import ClientSession, ContentTypeError, ClientError
 
 from mautrix.api import HTTPAPI, Method
-from mautrix.types import UserID, DeviceID, VersionsResponse, SerializerError
+from mautrix.errors import (
+    WellKnownInvalidVersionsResponse,
+    WellKnownMissingHomeserver,
+    WellKnownNotJSON,
+    WellKnownNotURL,
+    WellKnownUnexpectedStatus,
+    WellKnownUnsupportedScheme,
+)
+from mautrix.types import DeviceID, SerializerError, UserID, VersionsResponse
 from mautrix.util.logging import TraceLogger
-from mautrix.errors import (WellKnownNotURL, WellKnownNotJSON, WellKnownMissingHomeserver,
-                            WellKnownUnexpectedStatus, WellKnownUnsupportedScheme,
-                            WellKnownInvalidVersionsResponse)
 
 
 class BaseClientAPI:
@@ -32,20 +36,24 @@ class BaseClientAPI:
     _mxid: UserID
     device_id: DeviceID
     api: HTTPAPI
-    loop: asyncio.AbstractEventLoop
     log: TraceLogger
 
-    def __init__(self, mxid: UserID = "", device_id: DeviceID = "", api: HTTPAPI = None,
-                 loop: Optional[asyncio.AbstractEventLoop] = None, *args, **kwargs) -> None:
+    def __init__(
+        self, mxid: UserID = "", device_id: DeviceID = "", api: HTTPAPI | None = None, **kwargs
+    ) -> None:
         """
-        Initialize a ClientAPI. You must either provide the
+        Initialize a ClientAPI. You must either provide the ``api`` parameter with an existing
+        :class:`mautrix.api.HTTPAPI` instance, or provide the ``base_url`` and other arguments for
+        creating it as kwargs.
 
         Args:
             mxid: The Matrix ID of the user. This is used for things like setting profile metadata.
-                Additionally, the homeserver domain is extracted from this string and used for
-                setting aliases and such. This can be changed later using `set_mxid`.
-            api: The :class:`HTTPAPI` instance to use. You can also pass the ``args`` and ``kwargs``
-                to create a HTTPAPI instance rather than creating the instance yourself.``
+                  Additionally, the homeserver domain is extracted from this string and used for
+                  setting aliases and such. This can be changed later using `set_mxid`.
+            device_id: The device ID corresponding to the access token used.
+            api: The :class:`mautrix.api.HTTPAPI` instance to use. You can also pass the ``kwargs``
+                 to create a HTTPAPI instance rather than creating the instance yourself.
+            kwargs: If ``api`` is not specified, then the arguments to pass when creating a HTTPAPI.
         """
         if mxid:
             self.mxid = mxid
@@ -55,20 +63,11 @@ class BaseClientAPI:
             self.domain = None
         self.fill_member_event_callback = None
         self.device_id = device_id
-        if loop:
-            kwargs["loop"] = loop
-        self.api = api or HTTPAPI(*args, **kwargs)
-        self.loop = self.api.loop
+        self.api = api or HTTPAPI(**kwargs)
         self.log = self.api.log
 
     @classmethod
-    def parse_mxid(cls, mxid: UserID) -> Tuple[str, str]:
-        warnings.warn("parse_mxid is deprecated, use parse_user_id instead",
-                      category=DeprecationWarning)
-        return cls.parse_user_id(mxid)
-
-    @classmethod
-    def parse_user_id(cls, mxid: UserID) -> Tuple[str, str]:
+    def parse_user_id(cls, mxid: UserID) -> tuple[str, str]:
         """
         Parse the localpart and server name from a Matrix user ID.
 
@@ -91,7 +90,7 @@ class BaseClientAPI:
             raise ValueError("User ID must contain domain separator") from e
         if sep == len(mxid) - 1:
             raise ValueError("User ID must contain domain")
-        return mxid[1:sep], mxid[sep + 1:]
+        return mxid[1:sep], mxid[sep + 1 :]
 
     @property
     def mxid(self) -> UserID:
@@ -108,7 +107,7 @@ class BaseClientAPI:
         return VersionsResponse.deserialize(resp)
 
     @classmethod
-    async def discover(cls, domain: str, session: Optional[ClientSession] = None) -> Optional[URL]:
+    async def discover(cls, domain: str, session: ClientSession | None = None) -> URL | None:
         """
         Follow the server discovery spec to find the actual URL when given a Matrix server name.
 
@@ -130,7 +129,7 @@ class BaseClientAPI:
             return await cls._discover(domain, session)
 
     @classmethod
-    async def _discover(cls, domain: str, session: ClientSession) -> Optional[URL]:
+    async def _discover(cls, domain: str, session: ClientSession) -> URL | None:
         well_known = URL.build(scheme="https", host=domain, path="/.well-known/matrix/client")
         async with session.get(well_known) as resp:
             if resp.status == 404:
