@@ -3,37 +3,77 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Awaitable, Dict, Optional, List
+from __future__ import annotations
+
+from typing import Awaitable
 
 from mautrix.api import Method, Path
 from mautrix.errors import MatrixResponseError
-from mautrix.types import (JSON, UserID, RoomID, EventID, FilterID, SyncToken, PaginationDirection,
-                           StateEvent, EventType, StateEventContent, MessageEventContent, Member,
-                           Event, ContentURI, PaginatedMessages, SerializerError, MessageType,
-                           RelatesTo, Format, ImageInfo, BaseFileInfo, TextMessageEventContent,
-                           MediaMessageEventContent, PresenceState, EventContent, Membership,
-                           ReactionEventContent, RelationType, Obj, Serializable)
+from mautrix.types import (
+    JSON,
+    BaseFileInfo,
+    ContentURI,
+    Event,
+    EventContent,
+    EventID,
+    EventType,
+    FilterID,
+    Format,
+    ImageInfo,
+    MediaMessageEventContent,
+    Member,
+    Membership,
+    MessageEventContent,
+    MessageType,
+    Obj,
+    PaginatedMessages,
+    PaginationDirection,
+    PresenceState,
+    ReactionEventContent,
+    RelatesTo,
+    RelationType,
+    RoomID,
+    Serializable,
+    SerializerError,
+    StateEvent,
+    StateEventContent,
+    SyncToken,
+    TextMessageEventContent,
+    UserID,
+)
 from mautrix.types.event.state import state_event_content_map
+from mautrix.util.formatter import parse_html
+
 from .base import BaseClientAPI
 
 
 class EventMethods(BaseClientAPI):
     """
-    Methods in section 8 Events of the spec. Includes ``/sync``ing, getting messages and state,
+    Methods in section 8 Events of the spec. Includes ``/sync``'ing, getting messages and state,
     setting state, sending messages and redacting messages. See also: `Events API reference`_
 
     .. _Events API reference:
-        https://matrix.org/docs/spec/client_server/r0.5.0#events
+        https://spec.matrix.org/v1.1/client-server-api/#events
     """
 
-    # region 8.2 Syncing
-    # API reference: https://matrix.org/docs/spec/client_server/r0.4.0.html#syncing
+    # region 8.4 Syncing
+    # API reference: https://spec.matrix.org/v1.1/client-server-api/#syncing
 
-    def sync(self, since: SyncToken = None, timeout: int = 30000, filter_id: FilterID = None,
-             full_state: bool = False, set_presence: PresenceState = None) -> Awaitable[JSON]:
+    def sync(
+        self,
+        since: SyncToken | None = None,
+        timeout: int = 30000,
+        filter_id: FilterID | None = None,
+        full_state: bool = False,
+        set_presence: PresenceState | None = None,
+    ) -> Awaitable[JSON]:
         """
         Perform a sync request. See also: `/sync API reference`_
 
+        This method doesn't parse the response at all.
+        You should use :class:`mautrix.client.Syncer` to parse sync responses and dispatch the data
+        into event handlers. :class:`mautrix.client.Client` includes ``Syncer``.
+
         Args:
             since (str): Optional. A token which specifies where to continue a sync from.
             timeout (int): Optional. The time in milliseconds to wait.
@@ -43,7 +83,7 @@ class EventMethods(BaseClientAPI):
             set_presence (str): Should the client be marked as "online" or" offline"
 
         .. _/sync API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-sync
+            https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3sync
         """
         request = {"timeout": timeout}
         if since:
@@ -54,17 +94,20 @@ class EventMethods(BaseClientAPI):
             request["full_state"] = "true" if full_state else "false"
         if set_presence:
             request["set_presence"] = str(set_presence)
-        return self.api.request(Method.GET, Path.sync, query_params=request, retry_count=0)
+        return self.api.request(
+            Method.GET, Path.sync, query_params=request, retry_count=0, metrics_method="sync"
+        )
 
     # endregion
-    # region 8.3 Getting events for a room
-    # API reference: https://matrix.org/docs/spec/client_server/r0.4.0.html#getting-events-for-a-room
+    # region 7.5 Getting events for a room
+    # API reference: https://spec.matrix.org/v1.1/client-server-api/#getting-events-for-a-room
 
     async def get_event(self, room_id: RoomID, event_id: EventID) -> Event:
         """
         Get a single event based on ``room_id``/``event_id``. You must have permission to retrieve
         this event e.g. by being a member in the room for this event.
-        See also: `/event/{eventId} API reference`_
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomideventeventid>`__
 
         Args:
             room_id: The ID of the room the event is in.
@@ -72,23 +115,27 @@ class EventMethods(BaseClientAPI):
 
         Returns:
             The event.
-
-        .. _/event/{eventId} API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-event-eventid
         """
-        content = await self.api.request(Method.GET, Path.rooms[room_id].event[event_id])
+        content = await self.api.request(
+            Method.GET, Path.rooms[room_id].event[event_id], metrics_method="getEvent"
+        )
         try:
             return Event.deserialize(content)
         except SerializerError as e:
             raise MatrixResponseError("Invalid event in response") from e
 
-    async def get_state_event(self, room_id: RoomID, event_type: EventType,
-                              state_key: Optional[str] = None) -> StateEventContent:
+    async def get_state_event(
+        self,
+        room_id: RoomID,
+        event_type: EventType,
+        state_key: str = "",
+    ) -> StateEventContent:
         """
         Looks up the contents of a state event in a room. If the user is joined to the room then the
         state is taken from the current state of the room. If the user has left the room then the
         state is taken from the state of the room when they left.
-        See also: `GET /state/{eventType}/{stateKey} API reference`_
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidstateeventtypestatekey>`__
 
         Args:
             room_id: The ID of the room to look up the state in.
@@ -97,12 +144,12 @@ class EventMethods(BaseClientAPI):
 
         Returns:
             The state event.
-
-        .. _GET /state/{eventType}/{stateKey} API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-state-eventtype-statekey
         """
-        content = await self.api.request(Method.GET,
-                                         Path.rooms[room_id].state[event_type][state_key])
+        content = await self.api.request(
+            Method.GET,
+            Path.rooms[room_id].state[event_type][state_key],
+            metrics_method="getStateEvent",
+        )
         try:
             return state_event_content_map[event_type].deserialize(content)
         except KeyError:
@@ -110,30 +157,37 @@ class EventMethods(BaseClientAPI):
         except SerializerError as e:
             raise MatrixResponseError("Invalid state event in response") from e
 
-    async def get_state(self, room_id: RoomID) -> List[StateEvent]:
+    async def get_state(self, room_id: RoomID) -> list[StateEvent]:
         """
-        Get the state events for the current state of a room. See also: `/state API reference`_
+        Get the state events for the current state of a room.
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidstate>`__
 
         Args:
             room_id: The ID of the room to look up the state for.
 
         Returns:
             A list of state events with the most recent of each event_type/state_key pair.
-
-        .. _/state API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-state
         """
-        content = await self.api.request(Method.GET, Path.rooms[room_id].state)
+        content = await self.api.request(
+            Method.GET, Path.rooms[room_id].state, metrics_method="getState"
+        )
         try:
             return [StateEvent.deserialize(event) for event in content]
         except SerializerError as e:
             raise MatrixResponseError("Invalid state events in response") from e
 
-    async def get_members(self, room_id: RoomID, at: Optional[SyncToken] = None,
-                          membership: Optional[Membership] = None,
-                          not_membership: Optional[Membership] = None) -> List[StateEvent]:
+    async def get_members(
+        self,
+        room_id: RoomID,
+        at: SyncToken | None = None,
+        membership: Membership | None = None,
+        not_membership: Membership | None = None,
+    ) -> list[StateEvent]:
         """
-        Get the list of members for a room. See also: `/members API reference`_
+        Get the list of members for a room.
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidmembers>`__
 
         Args:
             room_id: The ID of the room to get the member events for.
@@ -149,9 +203,6 @@ class EventMethods(BaseClientAPI):
 
         Returns:
             A list of most recent member events for each user.
-
-        .. _/members API reference:
-            https://matrix.org/docs/spec/client_server/r0.6.1#get-matrix-client-r0-rooms-roomid-members
         """
         query = {}
         if at:
@@ -160,8 +211,12 @@ class EventMethods(BaseClientAPI):
             query["membership"] = membership.value
         if not_membership:
             query["not_membership"] = not_membership.value
-        content = await self.api.request(Method.GET, Path.rooms[room_id].members,
-                                         query_params=query)
+        content = await self.api.request(
+            Method.GET,
+            Path.rooms[room_id].members,
+            query_params=query,
+            metrics_method="getMembers",
+        )
         try:
             return [StateEvent.deserialize(event) for event in content["chunk"]]
         except KeyError:
@@ -169,13 +224,14 @@ class EventMethods(BaseClientAPI):
         except SerializerError as e:
             raise MatrixResponseError("Invalid state events in response") from e
 
-    async def get_joined_members(self, room_id: RoomID) -> Dict[UserID, Member]:
+    async def get_joined_members(self, room_id: RoomID) -> dict[UserID, Member]:
         """
         Get a user ID -> member info map for a room. The current user must be in the room for it to
         work, unless it is an Application Service in which case any of the AS's users must be in the
         room. This API is primarily for Application Services and should be faster to respond than
         `/members`_ as it can be implemented more efficiently on the server.
-        See also: `/joined_members API reference`_
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidjoined_members>`__
 
         Args:
             room_id: The ID of the room to get the members of.
@@ -183,29 +239,40 @@ class EventMethods(BaseClientAPI):
         Returns:
             A dictionary from user IDs to Member info objects.
 
-        .. _/joined_members API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-joined-members
         .. _/members:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-members
+            https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidmembers
         """
-        content = await self.api.request(Method.GET, Path.rooms[room_id].joined_members)
+        content = await self.api.request(
+            Method.GET, Path.rooms[room_id].joined_members, metrics_method="getJoinedMembers"
+        )
         try:
-            return {user_id: Member(membership=Membership.JOIN,
-                                    displayname=member.get("display_name", ""),
-                                    avatar_url=member.get("avatar_url", ""))
-                    for user_id, member in content["joined"].items()}
+            return {
+                user_id: Member(
+                    membership=Membership.JOIN,
+                    displayname=member.get("display_name", ""),
+                    avatar_url=member.get("avatar_url", ""),
+                )
+                for user_id, member in content["joined"].items()
+            }
         except KeyError:
             raise MatrixResponseError("`joined` not in response.")
         except SerializerError as e:
             raise MatrixResponseError("Invalid member objects in response") from e
 
-    async def get_messages(self, room_id: RoomID, direction: PaginationDirection,
-                           from_token: SyncToken, to_token: Optional[SyncToken] = None,
-                           limit: Optional[int] = None, filter_json: Optional[str] = None
-                           ) -> PaginatedMessages:
+    async def get_messages(
+        self,
+        room_id: RoomID,
+        direction: PaginationDirection,
+        from_token: SyncToken,
+        to_token: SyncToken | None = None,
+        limit: int | None = None,
+        filter_json: str | None = None,
+    ) -> PaginatedMessages:
         """
         Get a list of message and state events for a room. Pagination parameters are used to
-        paginate history in the room. See also: `/messages API reference`_
+        paginate history in the room.
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3roomsroomidmessages>`__
 
         Args:
             room_id: The ID of the room to get events from.
@@ -220,11 +287,9 @@ class EventMethods(BaseClientAPI):
         Returns:
 
         .. _RoomEventFilter:
-            https://matrix.org/docs/spec/client_server/r0.5.0#filtering
+            https://spec.matrix.org/v1.1/client-server-api/#filtering
         .. _sync endpoint:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-sync
-        .. _/messages API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-messages
+            https://spec.matrix.org/v1.1/client-server-api/#get_matrixclientv3sync
         """
         query_params = {
             "from": from_token,
@@ -233,11 +298,18 @@ class EventMethods(BaseClientAPI):
             "limit": str(limit) if limit else None,
             "filter_json": filter_json,
         }
-        content = await self.api.request(Method.GET, Path.rooms[room_id].messages,
-                                         query_params=query_params)
+        content = await self.api.request(
+            Method.GET,
+            Path.rooms[room_id].messages,
+            query_params=query_params,
+            metrics_method="getMessages",
+        )
         try:
-            return PaginatedMessages(content["start"], content["end"],
-                                     [Event.deserialize(event) for event in content["chunk"]])
+            return PaginatedMessages(
+                content["start"],
+                content["end"],
+                [Event.deserialize(event) for event in content["chunk"]],
+            )
         except KeyError:
             if "start" not in content:
                 raise MatrixResponseError("`start` not in response.")
@@ -248,16 +320,22 @@ class EventMethods(BaseClientAPI):
             raise MatrixResponseError("Invalid events in response") from e
 
     # endregion
-    # region 8.4 Sending events to a room
-    # API reference: https://matrix.org/docs/spec/client_server/r0.4.0.html#sending-events-to-a-room
+    # region 7.6 Sending events to a room
+    # API reference: https://spec.matrix.org/v1.1/client-server-api/#sending-events-to-a-room
 
-    async def send_state_event(self, room_id: RoomID, event_type: EventType,
-                               content: StateEventContent, state_key: Optional[str] = "", **kwargs
-                               ) -> EventID:
+    async def send_state_event(
+        self,
+        room_id: RoomID,
+        event_type: EventType,
+        content: StateEventContent,
+        state_key: str = "",
+        **kwargs,
+    ) -> EventID:
         """
         Send a state event to a room. State events with the same ``room_id``, ``event_type`` and
         ``state_key`` will be overridden.
-        See also: `PUT /state/{eventType}/{stateKey} API reference`_
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#put_matrixclientv3roomsroomidstateeventtypestatekey>`__
 
         Args:
             room_id: The ID of the room to set the state in.
@@ -270,40 +348,45 @@ class EventMethods(BaseClientAPI):
 
         Returns:
             The ID of the event that was sent.
-
-        .. _PUT /state/{eventType}/{stateKey} API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#put-matrix-client-r0-rooms-roomid-state-eventtype-statekey
         """
         content = content.serialize() if isinstance(content, Serializable) else content
-        resp = await self.api.request(Method.PUT, Path.rooms[room_id].state[event_type][state_key],
-                                      content, **kwargs)
+        resp = await self.api.request(
+            Method.PUT,
+            Path.rooms[room_id].state[event_type][state_key],
+            content,
+            **kwargs,
+            metrics_method="sendStateEvent",
+        )
         try:
             return resp["event_id"]
         except KeyError:
             raise MatrixResponseError("`event_id` not in response.")
 
-    async def send_message_event(self, room_id: RoomID, event_type: EventType,
-                                 content: EventContent, txn_id: Optional[str] = None,
-                                 **kwargs) -> EventID:
+    async def send_message_event(
+        self,
+        room_id: RoomID,
+        event_type: EventType,
+        content: EventContent,
+        txn_id: str | None = None,
+        **kwargs,
+    ) -> EventID:
         """
         Send a message event to a room. Message events allow access to historical events and
         pagination, making them suited for "once-off" activity in a room.
-        See also: `/send API reference`_
+
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#put_matrixclientv3roomsroomidsendeventtypetxnid>`__
 
         Args:
             room_id: The ID of the room to send the message to.
             event_type: The type of message to send.
             content: The content to send.
-            txn_id: The transaction ID to use.
+            txn_id: The transaction ID to use. If not provided, a random ID will be generated.
             **kwargs: Optional parameters to pass to the :meth:`HTTPAPI.request` method. Used by
                 :class:`IntentAPI` to pass the timestamp massaging field to
                 :meth:`AppServiceAPI.request`.
 
         Returns:
             The ID of the event that was sent.
-
-        .. _/send API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#put-matrix-client-r0-rooms-roomid-send-eventtype-txnid
         """
         if not room_id:
             raise ValueError("Room ID not given")
@@ -311,15 +394,22 @@ class EventMethods(BaseClientAPI):
             raise ValueError("Event type not given")
         url = Path.rooms[room_id].send[event_type][txn_id or self.api.get_txn_id()]
         content = content.serialize() if isinstance(content, Serializable) else content
-        resp = await self.api.request(Method.PUT, url, content, **kwargs)
+        resp = await self.api.request(
+            Method.PUT, url, content, **kwargs, metrics_method="sendMessageEvent"
+        )
         try:
             return resp["event_id"]
         except KeyError:
             raise MatrixResponseError("`event_id` not in response.")
 
     # region Message send helper functions
-    def send_message(self, room_id: RoomID, content: MessageEventContent, **kwargs
-                     ) -> Awaitable[EventID]:
+
+    def send_message(
+        self,
+        room_id: RoomID,
+        content: MessageEventContent,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send a message to a room.
 
@@ -334,41 +424,61 @@ class EventMethods(BaseClientAPI):
         return self.send_message_event(room_id, EventType.ROOM_MESSAGE, content, **kwargs)
 
     def react(self, room_id: RoomID, event_id: EventID, key: str, **kwargs) -> Awaitable[EventID]:
-        content = ReactionEventContent(relates_to=RelatesTo(rel_type=RelationType.ANNOTATION,
-                                                            event_id=event_id, key=key))
+        content = ReactionEventContent(
+            relates_to=RelatesTo(rel_type=RelationType.ANNOTATION, event_id=event_id, key=key)
+        )
         return self.send_message_event(room_id, EventType.REACTION, content, **kwargs)
 
-    def send_text(self, room_id: RoomID, text: str, html: Optional[str] = None,
-                  msgtype: MessageType = MessageType.TEXT, relates_to: Optional[RelatesTo] = None,
-                  **kwargs) -> Awaitable[EventID]:
+    async def send_text(
+        self,
+        room_id: RoomID,
+        text: str | None = None,
+        html: str | None = None,
+        msgtype: MessageType = MessageType.TEXT,
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> EventID:
         """
         Send a text message to a room.
 
         Args:
             room_id: The ID of the room to send the message to.
-            text: The text to send. If set to None, the given HTML is used instead.
+            text: The text to send. If set to ``None``, the given HTML will be parsed to generate
+                  a plaintext representation.
             html: The HTML to send.
             msgtype: The message type to send.
-                Defaults to :attr:`MessageType.TEXT` (normal text message)
+                     Defaults to :attr:`MessageType.TEXT` (normal text message).
             relates_to: Message relation metadata used for things like replies.
             **kwargs: Optional parameters to pass to the :meth:`HTTPAPI.request` method.
 
         Returns:
             The ID of the event that was sent.
+
+        Raises:
+            ValueError: if both ``text`` and ``html`` are ``None``.
         """
-        if html:
-            if not text:
-                text = html
-            content = TextMessageEventContent(msgtype=msgtype, body=text,
-                                              format=Format.HTML, formatted_body=html)
-        else:
+        if html is not None:
+            if text is None:
+                text = await parse_html(html)
+            content = TextMessageEventContent(
+                msgtype=msgtype, body=text, format=Format.HTML, formatted_body=html
+            )
+        elif text is not None:
             content = TextMessageEventContent(msgtype=msgtype, body=text)
+        else:
+            raise TypeError("send_text() requires either text or html to be set")
         if relates_to:
             content.relates_to = relates_to
-        return self.send_message(room_id, content, **kwargs)
+        return await self.send_message(room_id, content, **kwargs)
 
-    def send_notice(self, room_id: RoomID, text: str, html: Optional[str] = None,
-                    relates_to: Optional[RelatesTo] = None, **kwargs) -> Awaitable[EventID]:
+    def send_notice(
+        self,
+        room_id: RoomID,
+        text: str | None = None,
+        html: str | None = None,
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send a notice text message to a room. Notices are like normal text messages, but usually
         sent by bots to tell other bots not to react to them. If you're a bot, please send notices
@@ -376,41 +486,58 @@ class EventMethods(BaseClientAPI):
 
         Args:
             room_id: The ID of the room to send the message to.
-            text: The text to send. If set to None, the given HTML is used instead.
+            text: The text to send. If set to ``None``, the given HTML will be parsed to generate
+                  a plaintext representation.
             html: The HTML to send.
-            msgtype: The message type to send.
-                Defaults to :attr:`MessageType.TEXT` (normal text message)
             relates_to: Message relation metadata used for things like replies.
             **kwargs: Optional parameters to pass to the :meth:`HTTPAPI.request` method.
 
         Returns:
             The ID of the event that was sent.
+
+        Raises:
+            ValueError: if both ``text`` and ``html`` are ``None``.
         """
         return self.send_text(room_id, text, html, MessageType.NOTICE, relates_to, **kwargs)
 
-    def send_emote(self, room_id: RoomID, text: str, html: Optional[str] = None,
-                   relates_to: Optional[RelatesTo] = None, **kwargs) -> Awaitable[EventID]:
+    def send_emote(
+        self,
+        room_id: RoomID,
+        text: str | None = None,
+        html: str | None = None,
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send an emote to a room. Emotes are usually displayed by prepending a star and the user's
         display name to the message, which means they're usually written in the third person.
 
         Args:
             room_id: The ID of the room to send the message to.
-            text: The text to send. If set to None, the given HTML is used instead.
+            text: The text to send. If set to ``None``, the given HTML will be parsed to generate
+                  a plaintext representation.
             html: The HTML to send.
-            msgtype: The message type to send.
-                Defaults to :attr:`MessageType.TEXT` (normal text message)
             relates_to: Message relation metadata used for things like replies.
             **kwargs: Optional parameters to pass to the :meth:`HTTPAPI.request` method.
 
         Returns:
             The ID of the event that was sent.
+
+        Raises:
+            ValueError: if both ``text`` and ``html`` are ``None``.
         """
         return self.send_text(room_id, text, html, MessageType.EMOTE, relates_to, **kwargs)
 
-    def send_file(self, room_id: RoomID, url: ContentURI, info: Optional[BaseFileInfo] = None,
-                  file_name: str = None, file_type: MessageType = MessageType.FILE,
-                  relates_to: Optional[RelatesTo] = None, **kwargs) -> Awaitable[EventID]:
+    def send_file(
+        self,
+        room_id: RoomID,
+        url: ContentURI,
+        info: BaseFileInfo | None = None,
+        file_name: str | None = None,
+        file_type: MessageType = MessageType.FILE,
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send a file to a room.
 
@@ -429,14 +556,23 @@ class EventMethods(BaseClientAPI):
         Returns:
             The ID of the event that was sent.
         """
-        return self.send_message(room_id,
-                                 MediaMessageEventContent(url=url, info=info, body=file_name,
-                                                          relates_to=relates_to,
-                                                          msgtype=file_type), **kwargs)
+        return self.send_message(
+            room_id,
+            MediaMessageEventContent(
+                url=url, info=info, body=file_name, relates_to=relates_to, msgtype=file_type
+            ),
+            **kwargs,
+        )
 
-    def send_sticker(self, room_id: RoomID, url: ContentURI, info: Optional[ImageInfo],
-                     text: Optional[str] = "", relates_to: Optional[RelatesTo] = None, **kwargs
-                     ) -> Awaitable[EventID]:
+    def send_sticker(
+        self,
+        room_id: RoomID,
+        url: ContentURI,
+        info: ImageInfo | None = None,
+        text: str = "",
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send a sticker to a room. Stickers are basically images, but they're usually rendered
         slightly differently.
@@ -453,14 +589,22 @@ class EventMethods(BaseClientAPI):
         Returns:
             The ID of the event that was sent.
         """
-        return self.send_message_event(room_id, EventType.STICKER,
-                                       MediaMessageEventContent(url=url, info=info, body=text,
-                                                                relates_to=relates_to),
-                                       **kwargs)
+        return self.send_message_event(
+            room_id,
+            EventType.STICKER,
+            MediaMessageEventContent(url=url, info=info, body=text, relates_to=relates_to),
+            **kwargs,
+        )
 
-    def send_image(self, room_id: RoomID, url: ContentURI, info: Optional[ImageInfo] = None,
-                   file_name: str = None, relates_to: Optional[RelatesTo] = None, **kwargs
-                   ) -> Awaitable[EventID]:
+    def send_image(
+        self,
+        room_id: RoomID,
+        url: ContentURI,
+        info: ImageInfo | None = None,
+        file_name: str = None,
+        relates_to: RelatesTo | None = None,
+        **kwargs,
+    ) -> Awaitable[EventID]:
         """
         Send an image to a room.
 
@@ -476,17 +620,24 @@ class EventMethods(BaseClientAPI):
         Returns:
             The ID of the event that was sent.
         """
-        return self.send_file(room_id, url, info, file_name, MessageType.IMAGE, relates_to,
-                              **kwargs)
+        return self.send_file(
+            room_id, url, info, file_name, MessageType.IMAGE, relates_to, **kwargs
+        )
 
     # endregion
 
     # endregion
-    # region 8.5 Redactions
-    # API reference: https://matrix.org/docs/spec/client_server/r0.4.0.html#redactions
+    # region 7.7 Redactions
+    # API reference: https://spec.matrix.org/v1.1/client-server-api/#redactions
 
-    async def redact(self, room_id: RoomID, event_id: EventID, reason: Optional[str] = "", **kwargs
-                     ) -> EventID:
+    async def redact(
+        self,
+        room_id: RoomID,
+        event_id: EventID,
+        reason: str | None = None,
+        extra_content: dict[str, JSON] | None = None,
+        **kwargs,
+    ) -> EventID:
         """
         Send an event to redact a previous event.
 
@@ -498,24 +649,27 @@ class EventMethods(BaseClientAPI):
         Users may redact their own events, and any user with a power level greater than or equal to
         the redact power level of the room may redact events there.
 
-        See also: `/redact API reference`_
+        See also: `API reference <https://spec.matrix.org/v1.1/client-server-api/#put_matrixclientv3roomsroomidredacteventidtxnid>`__
 
         Args:
             room_id: The ID of the room the event is in.
             event_id: The ID of the event to redact.
             reason: The reason for the event being redacted.
+            extra_content: Extra content for the event.
             **kwargs: Optional parameters to pass to the :meth:`HTTPAPI.request` method. Used by
                 :class:`IntentAPI` to pass the timestamp massaging field to
                 :meth:`AppServiceAPI.request`.
 
         Returns:
             The ID of the event that was sent to redact the other event.
-
-        .. _/redact API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#put-matrix-client-r0-rooms-roomid-redact-eventid-txnid
         """
         url = Path.rooms[room_id].redact[event_id][self.api.get_txn_id()]
-        resp = await self.api.request(Method.PUT, url, content={"reason": reason}, **kwargs)
+        content = extra_content or {}
+        if reason:
+            content["reason"] = reason
+        resp = await self.api.request(
+            Method.PUT, url, content=content, **kwargs, metrics_method="redact"
+        )
         try:
             return resp["event_id"]
         except KeyError:
