@@ -1,4 +1,4 @@
 from .abstract import StateStore
-from .memory import MemoryStateStore
 from .file import FileStateStore
-from .sync import SyncStore, MemorySyncStore
+from .memory import MemoryStateStore
+from .sync import MemorySyncStore, SyncStore
