@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,30 +9,38 @@ from asyncpg import Connection
 
 from mautrix.util.async_db.upgrade import UpgradeTable
 
-upgrade_table = UpgradeTable(version_table_name="mx_version", database_name="matrix state cache",
-                             log=logging.getLogger("mau.client.db.upgrade"))
+upgrade_table = UpgradeTable(
+    version_table_name="mx_version",
+    database_name="matrix state cache",
+    log=logging.getLogger("mau.client.db.upgrade"),
+)
 
 
 @upgrade_table.register(description="Initial revision")
 async def upgrade_v1(conn: Connection, scheme: str) -> None:
-    await conn.execute("""CREATE TABLE mx_room_state (
+    await conn.execute(
+        """CREATE TABLE mx_room_state (
         room_id              VARCHAR(255) PRIMARY KEY,
         is_encrypted         BOOLEAN,
         has_full_member_list BOOLEAN,
         encryption           TEXT,
         power_levels         TEXT
-    )""")
+    )"""
+    )
     if scheme != "sqlite":
-        await conn.execute("CREATE TYPE membership AS ENUM "
-                           "('join', 'leave', 'invite', 'ban', 'knock')")
-    await conn.execute("""CREATE TABLE mx_user_profile (
+        await conn.execute(
+            "CREATE TYPE membership AS ENUM ('join', 'leave', 'invite', 'ban', 'knock')"
+        )
+    await conn.execute(
+        """CREATE TABLE mx_user_profile (
         room_id     VARCHAR(255),
         user_id     VARCHAR(255),
         membership  membership NOT NULL,
         displayname VARCHAR(255),
         avatar_url  VARCHAR(255),
         PRIMARY KEY (room_id, user_id)
-    )""")
+    )"""
+    )
 
 
 @upgrade_table.register(description="Stop using size-limited string fields")
