@@ -1,58 +1,70 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Any, Callable, Awaitable, Dict, TYPE_CHECKING
-import functools
+from __future__ import annotations
+
+from typing import Any, Awaitable, Callable, Dict
 import asyncio
+import functools
 import json
+import sys
 
 import olm
 
-from mautrix.types import (UserID, DeviceID, SigningKey, EncryptionKeyAlgorithm, SessionID,
-                           RequestedKeyInfo, RoomID, IdentityKey)
-from mautrix.client import Client
+from mautrix.types import (
+    DeviceID,
+    EncryptionKeyAlgorithm,
+    IdentityKey,
+    RequestedKeyInfo,
+    RoomID,
+    SessionID,
+    SigningKey,
+    UserID,
+)
 from mautrix.util.logging import TraceLogger
 
-from .types import DeviceIdentity
-
-if TYPE_CHECKING:
-    from .store import CryptoStore, StateStore
-    from .account import OlmAccount
+from .. import client as cli, crypto
 
+if sys.version_info >= (3, 8):
     from typing import TypedDict
+else:
+    from typing_extensions import TypedDict
 
 
-    class SignedObject(TypedDict):
-        signatures: Dict[UserID, Dict[str, str]]
-        unsigned: Any
+class SignedObject(TypedDict):
+    signatures: Dict[UserID, Dict[str, str]]
+    unsigned: Any
 
 
 class BaseOlmMachine:
-    client: Client
+    client: cli.Client
     log: TraceLogger
-    loop: asyncio.AbstractEventLoop
-    crypto_store: 'CryptoStore'
-    state_store: 'StateStore'
+    crypto_store: crypto.CryptoStore
+    state_store: crypto.StateStore
 
-    account: 'OlmAccount'
+    account: account.OlmAccount
 
     allow_unverified_devices: bool
     share_to_unverified_devices: bool
-    allow_key_share: Callable[[DeviceIdentity, RequestedKeyInfo], Awaitable[bool]]
+    allow_key_share: Callable[[crypto.DeviceIdentity, RequestedKeyInfo], Awaitable[bool]]
 
     # Futures that wait for responses to a key request
     _key_request_waiters: Dict[SessionID, asyncio.Future]
     # Futures that wait for a session to be received (either normally or through a key request)
     _inbound_session_waiters: Dict[SessionID, asyncio.Future]
 
-    async def wait_for_session(self, room_id: RoomID, sender_key: IdentityKey,
-                               session_id: SessionID, timeout: float = 3) -> bool:
+    _prev_unwedge: Dict[IdentityKey, float]
+
+    async def wait_for_session(
+        self, room_id: RoomID, sender_key: IdentityKey, session_id: SessionID, timeout: float = 3
+    ) -> bool:
         try:
             fut = self._inbound_session_waiters[session_id]
         except KeyError:
-            fut = self._inbound_session_waiters[session_id] = self.loop.create_future()
+            fut = asyncio.get_running_loop().create_future()
+            self._inbound_session_waiters[session_id] = fut
         try:
             return await asyncio.wait_for(asyncio.shield(fut), timeout)
         except asyncio.TimeoutError:
@@ -65,12 +77,14 @@ class BaseOlmMachine:
             return
 
 
-canonical_json = functools.partial(json.dumps, ensure_ascii=False, separators=(",", ":"),
-                                   sort_keys=True)
+canonical_json = functools.partial(
+    json.dumps, ensure_ascii=False, separators=(",", ":"), sort_keys=True
+)
 
 
-def verify_signature_json(data: 'SignedObject', user_id: UserID, device_id: DeviceID,
-                          key: SigningKey) -> bool:
+def verify_signature_json(
+    data: "SignedObject", user_id: UserID, device_id: DeviceID, key: SigningKey
+) -> bool:
     data_copy = {**data}
     data_copy.pop("unsigned", None)
     signatures = data_copy.pop("signatures")
