@@ -1,27 +1,37 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Dict, List
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 
-from mautrix.types import (IdentityKey, SessionID, RoomID, EventID, UserID, DeviceID,
-                           RoomEncryptionStateEventContent)
+from mautrix.types import (
+    DeviceID,
+    EventID,
+    IdentityKey,
+    RoomEncryptionStateEventContent,
+    RoomID,
+    SessionID,
+    UserID,
+)
 
-from .. import OlmAccount, Session, InboundGroupSession, OutboundGroupSession, DeviceIdentity
+from .. import DeviceIdentity, InboundGroupSession, OlmAccount, OutboundGroupSession, Session
 
 
 class StateStore(ABC):
     @abstractmethod
-    async def is_encrypted(self, room_id: RoomID) -> bool: ...
+    async def is_encrypted(self, room_id: RoomID) -> bool:
+        pass
 
     @abstractmethod
-    async def get_encryption_info(self, room_id: RoomID
-                                  ) -> Optional[RoomEncryptionStateEventContent]: ...
+    async def get_encryption_info(self, room_id: RoomID) -> RoomEncryptionStateEventContent | None:
+        pass
 
     @abstractmethod
-    async def find_shared_rooms(self, user_id: UserID) -> List[RoomID]: ...
+    async def find_shared_rooms(self, user_id: UserID) -> list[RoomID]:
+        pass
 
 
 class CryptoStore(ABC):
@@ -36,7 +46,7 @@ class CryptoStore(ABC):
     """The pickle key to use when pickling Olm objects."""
 
     @abstractmethod
-    async def get_device_id(self) -> Optional[DeviceID]:
+    async def get_device_id(self) -> DeviceID | None:
         """
         Get the device ID corresponding to this account_id
 
@@ -79,7 +89,7 @@ class CryptoStore(ABC):
         """Insert or update the OlmAccount in the store."""
 
     @abstractmethod
-    async def get_account(self) -> Optional[OlmAccount]:
+    async def get_account(self) -> OlmAccount | None:
         """Get the OlmAccount that was previously inserted with :meth:`put_account`.
         If no account has been inserted, this must return ``None``."""
 
@@ -97,7 +107,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_sessions(self, key: IdentityKey) -> List[Session]:
+    async def get_sessions(self, key: IdentityKey) -> list[Session]:
         """
         Get all Olm sessions in the store for the specific device.
 
@@ -110,7 +120,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_latest_session(self, key: IdentityKey) -> Optional[Session]:
+    async def get_latest_session(self, key: IdentityKey) -> Session | None:
         """
         Get the Olm session with the highest session ID (lexiographically sorting) for a specific
         device. It's usually safe to return the most recently added session if sorting by session
@@ -147,8 +157,13 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def put_group_session(self, room_id: RoomID, sender_key: IdentityKey,
-                                session_id: SessionID, session: InboundGroupSession) -> None:
+    async def put_group_session(
+        self,
+        room_id: RoomID,
+        sender_key: IdentityKey,
+        session_id: SessionID,
+        session: InboundGroupSession,
+    ) -> None:
         """
         Insert an inbound Megolm session into the store.
 
@@ -160,8 +175,9 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_group_session(self, room_id: RoomID, sender_key: IdentityKey,
-                                session_id: SessionID) -> Optional[InboundGroupSession]:
+    async def get_group_session(
+        self, room_id: RoomID, sender_key: IdentityKey, session_id: SessionID
+    ) -> InboundGroupSession | None:
         """
         Get an inbound Megolm group session that was previously inserted with
         :meth:`put_group_session`.
@@ -176,8 +192,9 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def has_group_session(self, room_id: RoomID, sender_key: IdentityKey,
-                                session_id: SessionID) -> bool:
+    async def has_group_session(
+        self, room_id: RoomID, sender_key: IdentityKey, session_id: SessionID
+    ) -> bool:
         """
         Check whether or not a specific inbound Megolm session is in the store. This is used before
         importing forwarded keys.
@@ -216,7 +233,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_outbound_group_session(self, room_id: RoomID) -> Optional[OutboundGroupSession]:
+    async def get_outbound_group_session(self, room_id: RoomID) -> OutboundGroupSession | None:
         """
         Get the stored outbound Megolm session from the store.
 
@@ -239,7 +256,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def remove_outbound_group_sessions(self, rooms: List[RoomID]) -> None:
+    async def remove_outbound_group_sessions(self, rooms: list[RoomID]) -> None:
         """
         Remove the stored outbound Megolm session for multiple rooms.
 
@@ -250,8 +267,14 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def validate_message_index(self, sender_key: IdentityKey, session_id: SessionID,
-                                     event_id: EventID, index: int, timestamp: int) -> bool:
+    async def validate_message_index(
+        self,
+        sender_key: IdentityKey,
+        session_id: SessionID,
+        event_id: EventID,
+        index: int,
+        timestamp: int,
+    ) -> bool:
         """
         Validate that a specific message isn't a replay attack.
 
@@ -276,7 +299,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_devices(self, user_id: UserID) -> Optional[Dict[DeviceID, DeviceIdentity]]:
+    async def get_devices(self, user_id: UserID) -> dict[DeviceID, DeviceIdentity] | None:
         """
         Get all devices for a given user.
 
@@ -290,7 +313,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def get_device(self, user_id: UserID, device_id: DeviceID) -> Optional[DeviceIdentity]:
+    async def get_device(self, user_id: UserID, device_id: DeviceID) -> DeviceIdentity | None:
         """
         Get a specific device identity.
 
@@ -303,7 +326,22 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def put_devices(self, user_id: UserID, devices: Dict[DeviceID, DeviceIdentity]) -> None:
+    async def find_device_by_key(
+        self, user_id: UserID, identity_key: IdentityKey
+    ) -> DeviceIdentity | None:
+        """
+        Find a specific device identity based on the identity key.
+
+        Args:
+            user_id: The ID of the user whose device to get.
+            identity_key: The identity key of the device to get.
+
+        Returns:
+            The :class:`DeviceIdentity` object, or ``None`` if not found.
+        """
+
+    @abstractmethod
+    async def put_devices(self, user_id: UserID, devices: dict[DeviceID, DeviceIdentity]) -> None:
         """
         Replace the stored device list for a specific user.
 
@@ -313,7 +351,7 @@ class CryptoStore(ABC):
         """
 
     @abstractmethod
-    async def filter_tracked_users(self, users: List[UserID]) -> List[UserID]:
+    async def filter_tracked_users(self, users: list[UserID]) -> list[UserID]:
         """
         Filter a list of user IDs to only include users whose device lists are being tracked.
 
