@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,8 +9,14 @@ from enum import IntEnum
 from attr import dataclass
 import attr
 
-from mautrix.types import (UserID, DeviceID, IdentityKey, SigningKey, SerializableAttrs,
-                           ToDeviceEvent)
+from mautrix.types import (
+    DeviceID,
+    IdentityKey,
+    SerializableAttrs,
+    SigningKey,
+    ToDeviceEvent,
+    UserID,
+)
 
 
 class TrustState(IntEnum):
