@@ -1,15 +1,12 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import TYPE_CHECKING
+from mautrix.types import IdentityKey, SessionID
 
 from .base import MatrixError
 
-if TYPE_CHECKING:
-    from mautrix.types import SessionID, IdentityKey
-
 
 class CryptoError(MatrixError):
     def __init__(self, message: str) -> None:
@@ -34,9 +31,10 @@ class MatchingSessionDecryptionError(DecryptionError):
 
 
 class SessionNotFound(DecryptionError):
-    def __init__(self, session_id: 'SessionID', sender_key: 'IdentityKey') -> None:
-        super().__init__("Failed to decrypt megolm event: "
-                         f"no session with given ID {session_id} found")
+    def __init__(self, session_id: SessionID, sender_key: IdentityKey) -> None:
+        super().__init__(
+            f"Failed to decrypt megolm event: no session with given ID {session_id} found"
+        )
         self.session_id = session_id
         self.sender_key = sender_key
 
