@@ -1,15 +1,16 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from typing import Dict, List, Union
+
 from attr import dataclass
 import attr
 
 from ..primitive import JSON, RoomID, UserID
-from ..util import SerializableAttrs, Obj, deserializer
-from .base import EventType, BaseEvent
+from ..util import Obj, SerializableAttrs, deserializer
+from .base import BaseEvent, EventType
 
 
 @dataclass
@@ -24,10 +25,10 @@ class RoomTagAccountDataEventContent(SerializableAttrs):
 
 DirectAccountDataEventContent = Dict[UserID, List[RoomID]]
 
-
 AccountDataEventContent = Union[RoomTagAccountDataEventContent, DirectAccountDataEventContent, Obj]
 account_data_event_content_map = {
     EventType.TAG: RoomTagAccountDataEventContent,
+    EventType.DIRECT: DirectAccountDataEventContent,
 }
 
 
@@ -39,7 +40,7 @@ class AccountDataEvent(BaseEvent, SerializableAttrs):
     content: AccountDataEventContent
 
     @classmethod
-    def deserialize(cls, data: JSON) -> 'AccountDataEvent':
+    def deserialize(cls, data: JSON) -> "AccountDataEvent":
         try:
             data.get("content", {})["__mautrix_event_type"] = EventType.find(data.get("type"))
         except ValueError:
