@@ -1,13 +1,14 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from typing import List, NamedTuple
+
 from attr import dataclass
 
 from .event import Membership
-from .primitive import UserID, ContentURI
+from .primitive import ContentURI, UserID
 from .util import SerializableAttrs
 
 
@@ -25,4 +26,6 @@ class User(SerializableAttrs):
     displayname: str = None
 
 
-UserSearchResults = NamedTuple("UserSearchResults", results=List[User], limit=int)
+class UserSearchResults(NamedTuple):
+    results: List[User]
+    limit: int
