@@ -1,9 +1,11 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Callable
+from __future__ import annotations
+
+from typing import Callable
 from abc import ABC
 
 from ruamel.yaml.comments import CommentedMap
@@ -13,9 +15,12 @@ from .recursive_dict import RecursiveDict
 
 
 class BaseProxyConfig(BaseConfig, ABC):
-    def __init__(self, load: Callable[[], CommentedMap],
-                 load_base: Callable[[], Optional[RecursiveDict[CommentedMap]]],
-                 save: Callable[[RecursiveDict[CommentedMap]], None]) -> None:
+    def __init__(
+        self,
+        load: Callable[[], CommentedMap],
+        load_base: Callable[[], RecursiveDict[CommentedMap] | None],
+        save: Callable[[RecursiveDict[CommentedMap]], None],
+    ) -> None:
         super().__init__()
         self._data = CommentedMap()
         self._load_proxy = load
